@@ -129,35 +129,38 @@ class _TarFileMetadataReader:
         if the prefix is an octal number indicating an incremental archive prefix.
         """
 
+        if '/' not in tarInfo.name:
+            return
+
         fixedPath = None
-        if '/' in tarInfo.name:
-            prefix, name = tarInfo.name.split('/', 1)
-            octalMTime = re.fullmatch("^[0-7]+$", prefix)
+        prefix, name = tarInfo.name.split('/', 1)
+        octalMTime = re.fullmatch("^[0-7]+$", prefix)
 
-            realPrefix = _TarFileMetadataReader._getTarPrefix(fileObject, tarInfo, printDebug)
+        realPrefix = _TarFileMetadataReader._getTarPrefix(fileObject, tarInfo, printDebug)
+        encodedPrefix = prefix.encode('utf8', 'surrogateescape')
 
-            # For names longer than 100B, GNU tar will store it using a ././@LongLink named file.
-            # In this case, tarfile will ignore the truncated filename AND the octal timestamp prefix!
-            # However, for long symbolic links, the prefix WILL be prepended to the @LongLink contents!
-            # In order to not strip folders erroneously, test against this prefix. Unfortunately, this is
-            # not perfect either because tarfile removes trailing slashes for names. So we have to
-            # read the TAR information ourselves.
-            # Note that the prefix contains two not always identical octal timestamps! E.g.,
-            #   b'13666753432\x0013666377326\x00\x00\x00...
-            # We only test for the first here as I'm not sure what the second one is.
-            if octalMTime and realPrefix and realPrefix.startswith(prefix.encode() + b"\0"):
-                secondTimestamp = re.match(b"^[0-7]+(\x00|)", realPrefix[len(prefix.encode()) + 1 :])
-                if secondTimestamp:
-                    fixedPath = name
-                elif printDebug >= 2:
-                    print("[Info] Second timestamp is not octal!", realPrefix[len(prefix.encode()) + 1 :])
+        # For names longer than 100B, GNU tar will store it using a ././@LongLink named file.
+        # In this case, tarfile will ignore the truncated filename AND the octal timestamp prefix!
+        # However, for long symbolic links, the prefix WILL be prepended to the @LongLink contents!
+        # In order to not strip folders erroneously, test against this prefix. Unfortunately, this is
+        # not perfect either because tarfile removes trailing slashes for names. So we have to
+        # read the TAR information ourselves.
+        # Note that the prefix contains two not always identical octal timestamps! E.g.,
+        #   b'13666753432\x0013666377326\x00\x00\x00...
+        # We only test for the first here as I'm not sure what the second one is.
+        if octalMTime and realPrefix and realPrefix.startswith(encodedPrefix + b"\0"):
+            secondTimestamp = re.match(b"^[0-7]+(\x00|)", realPrefix[len(encodedPrefix) + 1 :])
+            if secondTimestamp:
+                fixedPath = name
+            elif printDebug >= 2:
+                print("[Info] Second timestamp is not octal!", realPrefix[len(encodedPrefix) + 1 :])
 
-            if octalMTime and fixedPath is None and printDebug >= 1:
-                print(f"[Warning] ignored prefix '{prefix}' because it was not found in TAR header prefix.")
-                print("[Warning]", realPrefix[:30] if realPrefix else realPrefix)
-                print(f"[Info] TAR header offset: {tarInfo.offset}, type: {str(tarInfo.type)}")
-                print("[Info] name:", tarInfo.name)
-                print()
+        if octalMTime and fixedPath is None and printDebug >= 1:
+            print(f"[Warning] ignored prefix '{encodedPrefix!r}' because it was not found in TAR header prefix.")
+            print("[Warning]", realPrefix[:30] if realPrefix else realPrefix)
+            print(f"[Info] TAR header offset: {tarInfo.offset}, type: {str(tarInfo.type)}")
+            print("[Info] name:", tarInfo.name)
+            print()
 
         if fixedPath is not None:
             tarInfo.name = fixedPath
@@ -915,8 +918,8 @@ class SQLiteIndexedTar(MountSource):
             ]
 
             for _, cinfo in supportedCompressions.items():
-                if cinfo.moduleName in globals():
-                    module = globals()[cinfo.moduleName]
+                if cinfo.moduleName in sys.modules:
+                    module = sys.modules[cinfo.moduleName]
                     # zipfile has no __version__ attribute and PEP 396 ensuring that was rejected 2021-04-14
                     # in favor of 'version' from importlib.metadata which does not even work with zipfile.
                     # Probably, because zipfile is a built-in module whose version would be the Python version.
@@ -925,7 +928,7 @@ class SQLiteIndexedTar(MountSource):
                     # there is no generic way to get the "python-xz" name from the "xz" runtime module object
                     # and importlib.metadata.version will require "python-xz" as argument.
                     if hasattr(module, '__version__'):
-                        versions += [makeVersionRow(cinfo.moduleName, module.__version__)]
+                        versions += [makeVersionRow(cinfo.moduleName, getattr(module, '__version__'))]
 
             connection.executemany('INSERT OR REPLACE INTO "versions" VALUES (?,?,?,?,?)', versions)
         except Exception as exception:
@@ -1022,7 +1025,7 @@ class SQLiteIndexedTar(MountSource):
         sqlConnection = sqlite3.connect(path, **kwargs)
         sqlConnection.row_factory = sqlite3.Row
         sqlConnection.executescript(
-            # Looking mode exclusive leads to a measurable speedup. E.g., find on 2k recursive files tar
+            # Locking mode exclusive leads to a measurable speedup. E.g., find on 2k recursive files tar
             # improves from ~1s to ~0.4s!
             # https://blog.devart.com/increasing-sqlite-performance.html
             """
@@ -1569,6 +1572,14 @@ class SQLiteIndexedTar(MountSource):
         for row in rows:
             self._tryAddParentFolders(row[0], row[2], row[3])
 
+    @staticmethod
+    def _escapeInvalidCharacters(toEscape: str, encoding: str):
+        try:
+            toEscape.encode()
+            return toEscape
+        except UnicodeEncodeError:
+            return toEscape.encode(encoding, 'surrogateescape').decode(encoding, 'backslashreplace')
+
     def _setFileInfo(self, row: tuple) -> None:
         if not self.sqlConnection:
             raise IndexNotOpenError("This method can not be called without an opened index database!")
@@ -1586,16 +1597,7 @@ class SQLiteIndexedTar(MountSource):
             checkedRow = []
             for x in list(row):  # check strings
                 if isinstance(x, str):
-                    try:
-                        x.encode()
-                        checkedRow += [x]
-                    except UnicodeEncodeError:
-                        # fmt: off
-                        checkedRow += [
-                            x.encode( self.encoding, 'surrogateescape' )
-                             .decode( self.encoding, 'backslashreplace' )
-                         ]
-                        # fmt: on
+                    checkedRow += [self._escapeInvalidCharacters(x, self.encoding)]
                 else:
                     checkedRow += [x]
 
@@ -1605,6 +1607,9 @@ class SQLiteIndexedTar(MountSource):
             print("[Warning] The escaped inserted row is now:", row)
             print()
 
+            self._tryAddParentFolders(self._escapeInvalidCharacters(row[0], self.encoding), row[2], row[3])
+            return
+
         self._tryAddParentFolders(row[0], row[2], row[3])
 
     def indexIsLoaded(self) -> bool:
